<?php

namespace Pinpoint\PackingSlips\Controller\Adminhtml\Generate;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;

/**
 * Class BulkPrint
 * @package Pinpoint\PackingSlips\Controller\Adminhtml\Generate
 */
class BulkPrint extends Action
{
    /**
     * @var \Pinpoint\PackingSlips\Model\Pdf\GenerateSlip
     */
    protected $pickingSlip;

    /**
     * @var \iio\libmergepdf\Merger
     */
    protected $pdfMerger;

    /**
     * BulkPrint constructor.
     * @param Context $context
     * @param \Pinpoint\PackingSlips\Model\Pdf\GenerateSlip $pickingSlip
     * @param \iio\libmergepdf\Merger $merger
     */
    public function __construct(
        Context $context,
        \Pinpoint\PackingSlips\Model\Pdf\GenerateSlip $pickingSlip,
        \iio\libmergepdf\Merger $merger
    ) {
        $this->pickingSlip = $pickingSlip;
        $this->pdfMerger = $merger;

        parent::__construct($context);
    }

    /**
     * Check the permission to run it
     *
     * @return boolean
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('');
    }

    /**
     * @return bool|\Magento\Framework\App\ResponseInterface
     */
    public function execute()
    {
        $orderIds = $this->getRequest()->getParam('selected', []);
        $pdfs = [];
        $machines = ['Next Day Delivery' => []];

        //for ($i = 0; $i < 100; $i++) {
            foreach ($orderIds as $orderId) {
                $pdfInfo = $this->pickingSlip->generateSlip($orderId, false, true);

                /** @var \Magento\Sales\Model\Order $order */
                $order = $pdfInfo[2];

                if (strpos($order->getShippingDescription(), 'Next Day Delivery') !== false) {
                    $pdfInfo[1] = 'Next Day Delivery';
                }

                if (!isset($machines[$pdfInfo[1]])) {
                    $machines[$pdfInfo[1]] = [];
                }

                $machines[$pdfInfo[1]][] = $pdfInfo[0];
            }
        //}

        if (count($machines)) {
            //put simple (no personalisation, next day etc) products last
            if (isset($machines['simple'])) {
                $simpleOrders = $machines['simple'];
                unset($machines['simple']);
                $machines['simple'] = $simpleOrders;
            }

            if (!count($machines['Next Day Delivery'])) {
                unset($machines['Next Day Delivery']);
            }

            foreach ($machines as $machine => $orders) {
                $pdfs[] = $this->pickingSlip->createSplitterPage($machine);

                foreach ($orders as $order) {
                    $pdfs[] = $order;
                }
            }

            $merger = $this->pdfMerger;

            foreach ($pdfs as $pdf) {
                $merger->addRaw($pdf->output());
            }

            $content = $merger->merge();

            header('Content-Type: application/pdf');
            header('Content-Length: '.strlen( $content ));
            header('Content-disposition: attachment; filename="bulkprint-' . date('d-m-Y') . '.pdf"');
            header('Last-Modified: '.gmdate('D, d M Y H:i:s').' GMT');

            echo $content;
            return true;
        }

        return $this->_redirect('sales/order/index');
    }
}
